@@ -99,8 +99,8 @@ impl Configuration {
     /// If true, print `rerun-if-changed` directives to standard output, so that
     /// Cargo will only rerun the build script if any of the processed
     /// `.lalrpop` files are changed. This option is independent of
-    /// [`force_build`], although it would be usual to set [`force_build`] and
-    /// [`emit_rerun_directives`] at the same time.
+    /// [`Self::force_build()`], although it would be usual to set [`Self::force_build()`] and
+    /// [`Self::emit_rerun_directives()`] at the same time.
     ///
     /// While many build scripts will want to set this to `true`, the default is
     /// false, because emitting any rerun directives to Cargo will cause the
@@ -206,10 +206,7 @@ impl Configuration {
         }
 
         if session.out_dir.is_none() {
-            let out_dir = match env::var_os("OUT_DIR") {
-                Some(var) => var,
-                None => return Err("missing OUT_DIR variable")?,
-            };
+            let out_dir = env::var_os("OUT_DIR").ok_or("missing OUT_DIR variable")?;
             session.out_dir = Some(PathBuf::from(out_dir));
         }
 
@@ -219,15 +216,9 @@ impl Configuration {
                 env::vars()
                     .filter_map(|(feature_var, _)| {
                         let prefix = "CARGO_FEATURE_";
-                        if feature_var.starts_with(prefix) {
-                            Some(
-                                feature_var[prefix.len()..]
-                                    .replace("_", "-")
-                                    .to_ascii_lowercase(),
-                            )
-                        } else {
-                            None
-                        }
+                        feature_var
+                            .strip_prefix(prefix)
+                            .map(|feature| feature.replace('_', "-").to_ascii_lowercase())
                     })
                     .collect(),
             );
