@@ -4,7 +4,7 @@
 use crate::grammar::consts::{INPUT_LIFETIME, LALR, RECURSIVE_ASCENT, TABLE_DRIVEN, TEST_ALL};
 use crate::grammar::pattern::Pattern;
 use crate::grammar::repr::{self as r, NominalTypeRepr, TypeRepr};
-use crate::lexer::dfa::DFA;
+use crate::lexer::dfa::Dfa;
 use crate::message::builder::InlineBuilder;
 use crate::message::Content;
 use crate::tls::Tls;
@@ -28,10 +28,10 @@ pub struct Grammar {
 #[derive(Copy, Clone, Debug, Default, Hash, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Span(pub usize, pub usize);
 
-impl Into<Box<dyn Content>> for Span {
-    fn into(self) -> Box<dyn Content> {
+impl From<Span> for Box<dyn Content> {
+    fn from(val: Span) -> Self {
         let file_text = Tls::file_text();
-        let string = file_text.span_str(self);
+        let string = file_text.span_str(val);
 
         // Insert an Adjacent block to prevent wrapping inside this
         // string:
@@ -93,10 +93,7 @@ pub enum MatchItem {
 
 impl MatchItem {
     pub fn is_catch_all(&self) -> bool {
-        match *self {
-            MatchItem::CatchAll(_) => true,
-            _ => false,
-        }
+        matches!(*self, MatchItem::CatchAll(_))
     }
 
     pub fn span(&self) -> Span {
@@ -141,7 +138,7 @@ pub struct InternToken {
     /// Set of `r"foo"` and `"foo"` literals extracted from the
     /// grammar. Sorted by order of increasing precedence.
     pub match_entries: Vec<MatchEntry>,
-    pub dfa: DFA,
+    pub dfa: Dfa,
 }
 
 /// In `token_check`, as we prepare to generate a tokenizer, we
@@ -551,12 +548,12 @@ impl NonterminalString {
     }
 }
 
-impl Into<Box<dyn Content>> for NonterminalString {
-    fn into(self) -> Box<dyn Content> {
+impl From<NonterminalString> for Box<dyn Content> {
+    fn from(val: NonterminalString) -> Self {
         let session = Tls::session();
 
         InlineBuilder::new()
-            .text(self)
+            .text(val)
             .styled(session.nonterminal_symbol)
             .end()
     }
@@ -621,11 +618,11 @@ impl TerminalString {
     }
 }
 
-impl Into<Box<dyn Content>> for TerminalString {
-    fn into(self) -> Box<dyn Content> {
+impl From<TerminalString> for Box<dyn Content> {
+    fn from(val: TerminalString) -> Self {
         let session = Tls::session();
         InlineBuilder::new()
-            .text(self)
+            .text(val)
             .styled(session.terminal_symbol)
             .end()
     }
@@ -1203,13 +1200,13 @@ impl Path {
 
 pub fn read_algorithm(annotations: &[Annotation], algorithm: &mut r::Algorithm) {
     for annotation in annotations {
-        if annotation.id == Atom::from(LALR) {
+        if annotation.id == *LALR {
             algorithm.lalr = true;
-        } else if annotation.id == Atom::from(TABLE_DRIVEN) {
+        } else if annotation.id == *TABLE_DRIVEN {
             algorithm.codegen = r::LrCodeGeneration::TableDriven;
-        } else if annotation.id == Atom::from(RECURSIVE_ASCENT) {
+        } else if annotation.id == *RECURSIVE_ASCENT {
             algorithm.codegen = r::LrCodeGeneration::RecursiveAscent;
-        } else if annotation.id == Atom::from(TEST_ALL) {
+        } else if annotation.id == *TEST_ALL {
             algorithm.codegen = r::LrCodeGeneration::TestAll;
         } else {
             panic!(
