@@ -4,24 +4,27 @@
 #![allow(unused_parens)]
 // The builtin tests don't cover the CLI and so forth, and it's just
 // too darn annoying to try and make them do so.
-#![cfg_attr(test, allow(dead_code))]
+//
+// ε shows up in lalrpop/src/lr1/example/test.rs
+#![cfg_attr(test, allow(dead_code, mixed_script_confusables))]
 
 extern crate ascii_canvas;
-extern crate atty;
 extern crate bit_set;
 extern crate diff;
 extern crate ena;
+extern crate is_terminal;
 extern crate itertools;
-#[cfg_attr(feature = "test", macro_use)]
-extern crate lalrpop_util;
 extern crate petgraph;
 extern crate regex;
 extern crate regex_syntax;
-extern crate sha2;
 extern crate string_cache;
 extern crate term;
+extern crate tiny_keccak;
 extern crate unicode_xid;
 
+#[cfg_attr(feature = "test", macro_use)]
+extern crate lalrpop_util;
+
 #[cfg(test)]
 extern crate rand;
 
@@ -52,7 +55,7 @@ mod generate;
 #[cfg(test)]
 mod test_util;
 
-pub use api::process_root;
-pub use api::process_root_unconditionally;
-pub use api::Configuration;
+pub use crate::api::process_root;
+pub use crate::api::process_root_unconditionally;
+pub use crate::api::Configuration;
 use ascii_canvas::style;
