@@ -21,11 +21,11 @@ pub fn compile<'grammar, W: Write>(
     grammar: &'grammar Grammar,
     user_start_symbol: NonterminalString,
     start_symbol: NonterminalString,
-    states: &[LR1State<'grammar>],
+    states: &[Lr1State<'grammar>],
     action_module: &str,
     out: &mut RustWrite<W>,
 ) -> io::Result<()> {
-    let graph = StateGraph::new(&states);
+    let graph = StateGraph::new(states);
     let mut ascent = CodeGenerator::new_ascent(
         grammar,
         user_start_symbol,
@@ -94,7 +94,7 @@ impl<'grammar> StackSuffix<'grammar> {
     }
 
     /// returns the (optional, fixed) -- number of optional
-    /// items in stack prefix and numer of fixed
+    /// items in stack prefix and number of fixed
     fn optional_fixed_lens(&self) -> (usize, usize) {
         (self.len_optional, self.len() - self.len_optional)
     }
@@ -120,7 +120,7 @@ impl<'ascent, 'grammar, W: Write>
         user_start_symbol: NonterminalString,
         start_symbol: NonterminalString,
         graph: &'ascent StateGraph,
-        states: &'ascent [LR1State<'grammar>],
+        states: &'ascent [Lr1State<'grammar>],
         action_module: &str,
         out: &'ascent mut RustWrite<W>,
     ) -> Self {
@@ -153,7 +153,7 @@ impl<'ascent, 'grammar, W: Write>
     }
 
     /// Compute the stack suffix that the state expects on entry.
-    fn state_input_for(state: &'ascent LR1State<'grammar>) -> StackSuffix<'grammar> {
+    fn state_input_for(state: &'ascent Lr1State<'grammar>) -> StackSuffix<'grammar> {
         let max_prefix = state.max_prefix();
         let will_pop = state.will_pop();
         StackSuffix {
@@ -344,7 +344,7 @@ impl<'ascent, 'grammar, W: Write>
                     Token::Error => {
                         panic!("Error recovery is not implemented for recursive ascent parsers")
                     }
-                    Token::EOF => "None".to_string(),
+                    Token::Eof => "None".to_string(),
                 };
                 if index < tokens.len() - 1 {
                     rust!(self.out, "{} |", pattern);
@@ -373,7 +373,7 @@ impl<'ascent, 'grammar, W: Write>
                 || this_state
                     .reductions
                     .iter()
-                    .any(|&(ref t, _)| t.contains(&Token::Terminal(terminal.clone())))
+                    .any(|(t, _)| t.contains(&Token::Terminal(terminal.clone())))
         });
 
         rust!(self.out, "let {}expected = alloc::vec![", self.prefix);
@@ -434,7 +434,7 @@ impl<'ascent, 'grammar, W: Write>
 
         rust!(
             self.out,
-            "{}lalrpop_util::ParseError::UnrecognizedEOF {{",
+            "{}lalrpop_util::ParseError::UnrecognizedEof {{",
             self.prefix
         );
         rust!(self.out, "location: {}location,", self.prefix);
@@ -602,7 +602,7 @@ impl<'ascent, 'grammar, W: Write>
     //
     // Returns a list of argument names and a flag if this fn resulted
     // from pushing a terminal (in which case the lookahead must be
-    // computed interally).
+    // computed internally).
     fn fn_args(
         &mut self,
         optional_prefix: &[Symbol],
@@ -641,7 +641,7 @@ impl<'ascent, 'grammar, W: Write>
                 self.prefix,
                 i,
                 self.types
-                    .spanned_type(optional_prefix[i].ty(&self.types).clone()),
+                    .spanned_type(optional_prefix[i].ty(self.types).clone()),
             )
         });
 
@@ -652,7 +652,7 @@ impl<'ascent, 'grammar, W: Write>
                 self.prefix,
                 optional_prefix.len() + i,
                 self.types
-                    .spanned_type(fixed_prefix[i].ty(&self.types).clone())
+                    .spanned_type(fixed_prefix[i].ty(self.types).clone())
             )
         });
 
@@ -682,27 +682,22 @@ impl<'ascent, 'grammar, W: Write>
     ) -> io::Result<StackSuffix<'grammar>> {
         let mut result = inputs;
 
-        let top_opt = self
-            .custom
-            .graph
-            .successors(state_index)
-            .iter()
-            .any(|succ_state| {
-                let succ_inputs = &self.custom.state_inputs[succ_state.0];
+        let top_opt = self.custom.graph.successors(state_index).any(|succ_state| {
+            let succ_inputs = &self.custom.state_inputs[succ_state.0];
 
-                // Check for a successor state with a suffix like:
-                //
-                //     ... OPT_1 ... OPT_N FIXED_1
-                //
-                // (Remember that *every* successor state will have
-                // at least one fixed input.)
-                //
-                // So basically we are looking for states
-                // that, when they return, may *optionally* have consumed
-                // the top of our stack.
-                assert!(!succ_inputs.fixed().is_empty());
-                succ_inputs.fixed().len() == 1 && !succ_inputs.optional().is_empty()
-            });
+            // Check for a successor state with a suffix like:
+            //
+            //     ... OPT_1 ... OPT_N FIXED_1
+            //
+            // (Remember that *every* successor state will have
+            // at least one fixed input.)
+            //
+            // So basically we are looking for states
+            // that, when they return, may *optionally* have consumed
+            // the top of our stack.
+            assert!(!succ_inputs.fixed().is_empty());
+            succ_inputs.fixed().len() == 1 && !succ_inputs.optional().is_empty()
+        });
 
         // If we find a successor that may optionally consume the top
         // of our stack, convert our fixed inputs into optional ones.
@@ -851,13 +846,8 @@ impl<'ascent, 'grammar, W: Write>
         // reducing; but in the case of an empty production, it will come from the
         // lookahead or the end of the last symbol pushed
         if let (Some(first_sym), Some(last_sym)) = (transfer_syms.first(), transfer_syms.last()) {
-            rust!(
-                self.out,
-                "let {}start = {}.0.clone();",
-                self.prefix,
-                first_sym
-            );
-            rust!(self.out, "let {}end = {}.2.clone();", self.prefix, last_sym);
+            rust!(self.out, "let {}start = {}.0;", self.prefix, first_sym);
+            rust!(self.out, "let {}end = {}.2;", self.prefix, last_sym);
         } else if stack_suffix.len() > 0 {
             // we pop no symbols, so grab from the top of the stack
             // (unless we are in the start state)
@@ -890,10 +880,10 @@ impl<'ascent, 'grammar, W: Write>
             rust!(self.out, "let {p}end = {p}start;", p = self.prefix);
         }
 
-        let transfered_syms = transfer_syms.len();
+        let transferred_syms = transfer_syms.len();
 
         let mut args = transfer_syms;
-        if transfered_syms == 0 {
+        if transferred_syms == 0 {
             args.push(format!("&{}start", self.prefix));
             args.push(format!("&{}end", self.prefix));
         }
