@@ -21,7 +21,7 @@ pub fn compile<'grammar, W: Write>(
     grammar: &'grammar Grammar,
     user_start_symbol: NonterminalString,
     start_symbol: NonterminalString,
-    states: &[LR1State<'grammar>],
+    states: &[Lr1State<'grammar>],
     action_module: &str,
     out: &mut RustWrite<W>,
 ) -> io::Result<()> {
@@ -81,7 +81,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         grammar: &'grammar Grammar,
         user_start_symbol: NonterminalString,
         start_symbol: NonterminalString,
-        states: &'ascent [LR1State<'grammar>],
+        states: &'ascent [Lr1State<'grammar>],
         action_module: &str,
         out: &'ascent mut RustWrite<W>,
     ) -> Self {
@@ -306,6 +306,20 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         rust!(self.out, "{p}expected_tokens(state)", p = self.prefix);
         rust!(self.out, "}}");
 
+        rust!(self.out, "");
+        rust!(
+            self.out,
+            "fn expected_tokens_from_states(&self, states: &[{state_type}]) -> alloc::vec::Vec<alloc::string::String> {{",
+            state_type = state_type,
+        );
+        rust!(
+            self.out,
+            "{p}expected_tokens_from_states(states, {pde})",
+            p = self.prefix,
+            pde = phantom_data_expr,
+        );
+        rust!(self.out, "}}");
+
         rust!(self.out, "");
         rust!(self.out, "#[inline]");
         rust!(self.out, "fn uses_error_recovery(&self) -> bool {{");
@@ -378,19 +392,12 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
             p = self.prefix,
             state_type = state_type,
         );
-        if self.grammar.uses_error_recovery {
-            rust!(
-                self.out,
-                "{p}simulate_reduce(action, {phantom})",
-                p = self.prefix,
-                phantom = phantom_data_expr,
-            );
-        } else {
-            rust!(
-                self.out,
-                "panic!(\"error recovery not enabled for this grammar\")"
-            )
-        }
+        rust!(
+            self.out,
+            "{p}simulate_reduce(action, {phantom})",
+            p = self.prefix,
+            phantom = phantom_data_expr,
+        );
         rust!(self.out, "}}");
 
         rust!(self.out, "}}");
@@ -485,7 +492,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
             // Write an action for each terminal (either shift, reduce, or error).
             let custom = &self.custom;
             let iterator = self.grammar.terminals.all.iter().map(|terminal| {
-                if let Some(new_state) = state.shifts.get(&terminal) {
+                if let Some(new_state) = state.shifts.get(terminal) {
                     (
                         new_state.0 as i32 + 1,
                         Comment::Goto(Token::Terminal(terminal.clone()), new_state.0),
@@ -524,7 +531,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         );
         for (index, state) in self.states.iter().enumerate() {
             rust!(self.out, "// State {}", index);
-            let reduction = Self::write_reduction(&self.custom, state, &Token::EOF);
+            let reduction = Self::write_reduction(&self.custom, state, &Token::Eof);
             self.out.write_table_row(Some(reduction))?;
         }
         rust!(self.out, "];");
@@ -556,7 +563,9 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
 
         rust!(self.out, "}}");
 
+        self.emit_terminal_repr_list()?;
         self.emit_expected_tokens_fn()?;
+        self.emit_expected_tokens_from_states_fn()?;
 
         Ok(())
     }
@@ -664,13 +673,13 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
 
     fn write_reduction<'s>(
         custom: &TableDriven<'grammar>,
-        state: &'s LR1State,
+        state: &'s Lr1State,
         token: &Token,
     ) -> (i32, Comment<'s, Token>) {
         let reduction = state
             .reductions
             .iter()
-            .filter(|&&(ref t, _)| t.contains(token))
+            .filter(|&(t, _)| t.contains(token))
             .map(|&(_, p)| p)
             .next();
         if let Some(production) = reduction {
@@ -903,7 +912,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
 
         rust!(
             self.out,
-            "let ({p}pop_states, {p}nonterminal) = match {}action {{",
+            "let ({p}pop_states, {p}nonterminal) = match {p}action {{",
             p = self.prefix
         );
         for (production, index) in self
@@ -1065,13 +1074,8 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         // reducing; but in the case of an empty production, it will come from the
         // lookahead
         if let (Some(first_sym), Some(last_sym)) = (transfer_syms.first(), transfer_syms.last()) {
-            rust!(
-                self.out,
-                "let {}start = {}.0.clone();",
-                self.prefix,
-                first_sym
-            );
-            rust!(self.out, "let {}end = {}.2.clone();", self.prefix, last_sym);
+            rust!(self.out, "let {}start = {}.0;", self.prefix, first_sym);
+            rust!(self.out, "let {}end = {}.2;", self.prefix, last_sym);
         } else {
             // we pop no symbols, so grab from the top of the stack
             // (unless we are in the start state, in which case the
@@ -1084,10 +1088,10 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
             rust!(self.out, "let {p}end = {p}start.clone();", p = self.prefix,);
         }
 
-        let transfered_syms = transfer_syms.len();
+        let transferred_syms = transfer_syms.len();
 
         let mut args = transfer_syms;
-        if transfered_syms == 0 {
+        if transferred_syms == 0 {
             args.push(format!("&{}start", self.prefix));
             args.push(format!("&{}end", self.prefix));
         }
@@ -1225,9 +1229,6 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
     }
 
     fn write_simulate_reduce_fn(&mut self) -> io::Result<()> {
-        if !self.grammar.uses_error_recovery {
-            return Ok(());
-        }
         let state_type = self.custom.state_type;
 
         let parameters = vec![
@@ -1321,7 +1322,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
     ///
     /// ```ignore
     /// fn __accepts() {
-    ///     error_state: i32,
+    ///     error_state: Option<i32>,
     ///     states: &Vec<i32>,
     ///     opt_integer: Option<usize>,
     /// ) -> bool {
@@ -1329,9 +1330,10 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
     /// }
     /// ```
     ///
-    /// has the job of figuring out whether the given error state would
-    /// "accept" the given lookahead. We basically trace through the LR
-    /// automaton looking for one of two outcomes:
+    /// has the job of figuring out whether the given state stack (with the
+    /// optional error state appended) would "accept" the given lookahead. We
+    /// basically trace through the LR automaton looking for one of two
+    /// outcomes:
     ///
     /// - the lookahead is eventually shifted
     /// - we reduce to the end state successfully (in the case of EOF).
@@ -1345,19 +1347,15 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
     /// infinite loop (see the `error_recovery_lalr_loop` test) or
     /// produce crappy results (see `error_recovery_lock_in`).
     fn write_accepts_fn(&mut self) -> io::Result<()> {
-        if !self.grammar.uses_error_recovery {
-            return Ok(());
-        }
-
         let phantom_data_expr = self.phantom_data_expr();
         let parameters = vec![
             format!(
-                "{p}error_state: {typ}",
+                "{p}error_state: Option<{typ}>",
                 p = self.prefix,
                 typ = self.custom.state_type
             ),
             format!(
-                "{p}states: & [{typ}]",
+                "{p}states: &[{typ}]",
                 p = self.prefix,
                 typ = self.custom.state_type
             ),
@@ -1367,7 +1365,8 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
 
         self.out
             .fn_header(&Visibility::Priv, format!("{}accepts", self.prefix))
-            .with_grammar(self.grammar)
+            .with_type_parameters(&self.custom.machine.type_parameters)
+            .with_where_clauses(&self.custom.machine.where_clauses)
             .with_parameters(parameters)
             .with_return_type("bool")
             .emit()?;
@@ -1388,7 +1387,11 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
             "let mut {p}states = {p}states.to_vec();",
             p = self.prefix
         );
-        rust!(self.out, "{p}states.push({p}error_state);", p = self.prefix);
+        rust!(
+            self.out,
+            "{p}states.extend({p}error_state);",
+            p = self.prefix
+        );
 
         rust!(self.out, "loop {{",);
 
@@ -1517,17 +1520,11 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         format!("({},{},{})", loc_type, self.symbol_type(), loc_type)
     }
 
-    fn emit_expected_tokens_fn(&mut self) -> io::Result<()> {
-        rust!(
-            self.out,
-            "fn {p}expected_tokens({p}state: {}) -> alloc::vec::Vec<alloc::string::String> {{",
-            self.custom.state_type,
-            p = self.prefix,
-        );
-
+    /// Emit the array of terminal tokens for use in generating error output
+    fn emit_terminal_repr_list(&mut self) -> io::Result<()> {
         rust!(self.out, "const {}TERMINAL: &[&str] = &[", self.prefix);
         let all_terminals = if self.grammar.uses_error_recovery {
-            // Subtract one to exlude the error terminal
+            // Subtract one to exclude the error terminal
             &self.grammar.terminals.all[..self.grammar.terminals.all.len() - 1]
         } else {
             &self.grammar.terminals.all
@@ -1538,8 +1535,18 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
             rust!(self.out, "r###\"{}\"###,", terminal);
         }
         rust!(self.out, "];");
+        Ok(())
+    }
 
-        // Grab any terminals in the current state which would have resulted in a successful parse
+    fn emit_expected_tokens_fn(&mut self) -> io::Result<()> {
+        rust!(
+            self.out,
+            "fn {p}expected_tokens({p}state: {}) -> alloc::vec::Vec<alloc::string::String> {{",
+            self.custom.state_type,
+            p = self.prefix,
+        );
+
+        // Grab any terminals in the current state which could have resulted in a successful parse
         rust!(
             self.out,
             "{}TERMINAL.iter().enumerate().filter_map(|(index, terminal)| {{",
@@ -1553,7 +1560,59 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TableDrive
         rust!(self.out, "if next_state == 0 {{");
         rust!(self.out, "None");
         rust!(self.out, "}} else {{");
-        rust!(self.out, "Some(alloc::string::ToString::to_string(terminal))");
+        rust!(
+            self.out,
+            "Some(alloc::string::ToString::to_string(terminal))"
+        );
+        rust!(self.out, "}}");
+        rust!(self.out, "}}).collect()");
+        rust!(self.out, "}}");
+
+        Ok(())
+    }
+
+    fn emit_expected_tokens_from_states_fn(&mut self) -> io::Result<()> {
+        let parameters = vec![
+            format!(
+                "{p}states: &[{typ}]",
+                p = self.prefix,
+                typ = self.custom.state_type
+            ),
+            format!("_: {}", self.phantom_data_type()),
+        ];
+
+        self.out
+            .fn_header(
+                &Visibility::Priv,
+                format!("{}expected_tokens_from_states", self.prefix),
+            )
+            .with_type_parameters(&self.custom.machine.type_parameters)
+            .with_where_clauses(&self.custom.machine.where_clauses)
+            .with_parameters(parameters)
+            .with_return_type("alloc::vec::Vec<alloc::string::String>")
+            .emit()?;
+
+        rust!(self.out, "{{");
+
+        // Grab any terminals in the current state which would have resulted in a successful parse,
+        // as verified using accepts()
+        rust!(
+            self.out,
+            "{}TERMINAL.iter().enumerate().filter_map(|(index, terminal)| {{",
+            self.prefix,
+        );
+        rust!(
+            self.out,
+            "if {p}accepts(None, {p}states, Some(index), {pde}) {{",
+            p = self.prefix,
+            pde = self.phantom_data_expr(),
+        );
+        rust!(
+            self.out,
+            "Some(alloc::string::ToString::to_string(terminal))"
+        );
+        rust!(self.out, "}} else {{");
+        rust!(self.out, "None");
         rust!(self.out, "}}");
         rust!(self.out, "}}).collect()");
         rust!(self.out, "}}");
