@@ -5,8 +5,8 @@
 use crate::grammar::repr::{Grammar, NonterminalString, TypeParameter};
 use crate::lr1::core::*;
 use crate::rust::RustWrite;
-use std::io::{self, Write};
 use crate::util::Sep;
+use std::io::{self, Write};
 
 use super::base::CodeGenerator;
 
@@ -14,7 +14,7 @@ pub fn compile<'grammar, W: Write>(
     grammar: &'grammar Grammar,
     user_start_symbol: NonterminalString,
     start_symbol: NonterminalString,
-    states: &[LR1State<'grammar>],
+    states: &[Lr1State<'grammar>],
     out: &mut RustWrite<W>,
 ) -> io::Result<()> {
     let mut ascent =
@@ -29,7 +29,7 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TestAll> {
         grammar: &'grammar Grammar,
         user_start_symbol: NonterminalString,
         start_symbol: NonterminalString,
-        states: &'ascent [LR1State<'grammar>],
+        states: &'ascent [Lr1State<'grammar>],
         out: &'ascent mut RustWrite<W>,
     ) -> Self {
         CodeGenerator::new(
@@ -123,12 +123,10 @@ impl<'ascent, 'grammar, W: Write> CodeGenerator<'ascent, 'grammar, W, TestAll> {
             .collect();
         let parameters = if non_lifetimes.is_empty() {
             String::new()
+        } else if self.grammar.intern_token.is_some() {
+            format!("::<{}>", Sep(", ", &non_lifetimes))
         } else {
-            if self.grammar.intern_token.is_some() {
-                format!("::<{}>", Sep(", ", &non_lifetimes))
-            } else {
-                format!("::<{}, _, _>", Sep(", ", &non_lifetimes))
-            }
+            format!("::<{}, _, _>", Sep(", ", &non_lifetimes))
         };
         rust!(
             self.out,
