@@ -56,7 +56,7 @@ pub struct TraceGraph<'grammar> {
 #[derive(Clone, Debug, PartialOrd, Ord, PartialEq, Eq)]
 pub enum TraceGraphNode<'grammar> {
     Nonterminal(NonterminalString),
-    Item(LR0Item<'grammar>),
+    Item(Lr0Item<'grammar>),
 }
 
 impl<'grammar> TraceGraph<'grammar> {
@@ -97,7 +97,7 @@ impl<'grammar> TraceGraph<'grammar> {
 
     pub fn lr0_examples<'graph>(
         &'graph self,
-        lr0_item: LR0Item<'grammar>,
+        lr0_item: Lr0Item<'grammar>,
     ) -> PathEnumerator<'graph, 'grammar> {
         PathEnumerator::new(self, lr0_item)
     }
@@ -105,27 +105,27 @@ impl<'grammar> TraceGraph<'grammar> {
     pub fn lr1_examples<'trace>(
         &'trace self,
         first_sets: &'trace FirstSets,
-        item: &LR1Item<'grammar>,
+        item: &Lr1Item<'grammar>,
     ) -> FilteredPathEnumerator<'trace, 'grammar> {
         FilteredPathEnumerator::new(first_sets, self, item.to_lr0(), item.lookahead.clone())
     }
 }
 
-impl<'grammar> Into<TraceGraphNode<'grammar>> for NonterminalString {
-    fn into(self) -> TraceGraphNode<'grammar> {
-        TraceGraphNode::Nonterminal(self)
+impl<'grammar> From<NonterminalString> for TraceGraphNode<'grammar> {
+    fn from(val: NonterminalString) -> Self {
+        TraceGraphNode::Nonterminal(val)
     }
 }
 
-impl<'grammar, L: Lookahead> Into<TraceGraphNode<'grammar>> for Item<'grammar, L> {
-    fn into(self) -> TraceGraphNode<'grammar> {
-        (&self).into()
+impl<'grammar, L: Lookahead> From<Item<'grammar, L>> for TraceGraphNode<'grammar> {
+    fn from(val: Item<'grammar, L>) -> Self {
+        (&val).into()
     }
 }
 
-impl<'a, 'grammar, L: Lookahead> Into<TraceGraphNode<'grammar>> for &'a Item<'grammar, L> {
-    fn into(self) -> TraceGraphNode<'grammar> {
-        TraceGraphNode::Item(self.to_lr0())
+impl<'a, 'grammar, L: Lookahead> From<&'a Item<'grammar, L>> for TraceGraphNode<'grammar> {
+    fn from(val: &'a Item<'grammar, L>) -> Self {
+        TraceGraphNode::Item(val.to_lr0())
     }
 }
 
@@ -183,7 +183,7 @@ struct EnumeratorState<'graph, 'grammar: 'graph> {
 }
 
 impl<'graph, 'grammar> PathEnumerator<'graph, 'grammar> {
-    fn new(graph: &'graph TraceGraph<'grammar>, lr0_item: LR0Item<'grammar>) -> Self {
+    fn new(graph: &'graph TraceGraph<'grammar>, lr0_item: Lr0Item<'grammar>) -> Self {
         let start_state = graph.indices[&TraceGraphNode::Item(lr0_item)];
         let mut enumerator = PathEnumerator {
             graph,
@@ -427,7 +427,7 @@ impl<'graph, 'grammar> FilteredPathEnumerator<'graph, 'grammar> {
     fn new(
         first_sets: &'graph FirstSets,
         graph: &'graph TraceGraph<'grammar>,
-        lr0_item: LR0Item<'grammar>,
+        lr0_item: Lr0Item<'grammar>,
         lookahead: TokenSet,
     ) -> Self {
         FilteredPathEnumerator {
