@@ -181,12 +181,11 @@ impl<'grammar> Validator<'grammar> {
         Ok(())
     }
 
-    fn validate_precedence(&self, alternatives: &Vec<Alternative>) -> NormResult<()> {
+    fn validate_precedence(&self, alternatives: &[Alternative]) -> NormResult<()> {
         let with_precedence = alternatives.iter().any(|alt| {
-            alt.annotations.iter().any(|ann| {
-                ann.id == Atom::from(precedence::PREC_ANNOT)
-                    || ann.id == Atom::from(precedence::ASSOC_ANNOT)
-            })
+            alt.annotations
+                .iter()
+                .any(|ann| ann.id == *precedence::PREC_ANNOT || ann.id == *precedence::ASSOC_ANNOT)
         });
 
         if alternatives.is_empty() || !with_precedence {
@@ -204,7 +203,7 @@ impl<'grammar> Validator<'grammar> {
                 let ann_prec_opt = first
                     .annotations
                     .iter()
-                    .find(|ann| ann.id == Atom::from(precedence::PREC_ANNOT));
+                    .find(|ann| ann.id == *precedence::PREC_ANNOT);
 
                 if ann_prec_opt.is_none() {
                     return_err!(
@@ -219,19 +218,19 @@ impl<'grammar> Validator<'grammar> {
 
         // Check that annotations are well-formed
         alternatives.iter().try_for_each(|alt| {
-            let ann_prec_opt = alt.annotations.iter().find(|ann| ann.id == Atom::from(precedence::PREC_ANNOT));
-            let ann_assoc_opt = alt.annotations.iter().find(|ann| ann.id == Atom::from(precedence::ASSOC_ANNOT));
+            let ann_prec_opt = alt.annotations.iter().find(|ann| ann.id == *precedence::PREC_ANNOT);
+            let ann_assoc_opt = alt.annotations.iter().find(|ann| ann.id == *precedence::ASSOC_ANNOT);
 
             if let Some(ann_prec) = ann_prec_opt {
                 match &ann_prec.arg {
-                    Some((name, value)) if *name == Atom::from(precedence::LVL_ARG) => {
+                    Some((name, value)) if name == &Atom::from(precedence::LVL_ARG) => {
                         if let Ok(lvl) = value.parse::<u32>() {
                             if lvl < min_lvl {
                                 min_lvl = lvl;
-                                min_prec_ann = ann_assoc_opt.clone();
+                                min_prec_ann = ann_assoc_opt;
                             }
                             else if lvl == min_lvl && min_prec_ann.is_none() && ann_assoc_opt.is_some() {
-                                min_prec_ann = ann_assoc_opt.clone();
+                                min_prec_ann = ann_assoc_opt;
                             }
                         }
                         else {
@@ -245,7 +244,7 @@ impl<'grammar> Validator<'grammar> {
 
             if let Some(ann_assoc) = ann_assoc_opt {
                 match &ann_assoc.arg {
-                    Some((name, value)) if *name == Atom::from(precedence::SIDE_ARG) => {
+                    Some((name, value)) if name == &Atom::from(precedence::SIDE_ARG) => {
                         if value.parse::<precedence::Assoc>().is_err() {
                             return_err!(ann_assoc.id_span, "could not parse the associativity `{}`, expected `left`, `right`, `none` or `all`", value);
                         }
@@ -326,10 +325,7 @@ impl<'grammar> Validator<'grammar> {
         let chosen: Vec<&Symbol> = expr
             .symbols
             .iter()
-            .filter(|sym| match sym.kind {
-                SymbolKind::Choose(_) => true,
-                _ => false,
-            })
+            .filter(|sym| matches!(sym.kind, SymbolKind::Choose(_)))
             .collect();
 
         let named: Multimap<Atom, Vec<&Symbol>> = expr
