@@ -2,7 +2,7 @@ use crate::parser;
 use crate::test_util;
 
 fn check_err(expected_err: &str, grammar: &str, span: &str) {
-    let parsed_grammar = parser::parse_grammar(&grammar).unwrap();
+    let parsed_grammar = parser::parse_grammar(grammar).unwrap();
     let err = super::validate(&parsed_grammar).unwrap_err();
     test_util::check_norm_err(expected_err, span, err);
 }
