@@ -10,8 +10,8 @@ use super::{NormError, NormResult};
 use crate::collections::{Map, Set};
 use crate::grammar::consts::*;
 use crate::grammar::parse_tree::*;
-use crate::lexer::dfa::{self, DFAConstructionError, Precedence};
-use crate::lexer::nfa::NFAConstructionError::*;
+use crate::lexer::dfa::{self, DfaConstructionError, Precedence};
+use crate::lexer::nfa::NfaConstructionError::*;
 use crate::lexer::re;
 use string_cache::DefaultAtom as Atom;
 
@@ -34,12 +34,6 @@ pub fn validate(mut grammar: Grammar) -> NormResult<Grammar> {
                     .collect(),
             }
         } else {
-            if cfg!(not(feature = "lexer")) {
-                return_err!(
-                    Span::default(),
-                    "The `lexer` feature must be specified unless an `extern` lexer is defined"
-                );
-            }
             TokenMode::Internal {
                 match_block: MatchBlock::new(grammar.match_token())?,
             }
@@ -314,7 +308,7 @@ impl<'grammar> Validator<'grammar> {
 
 ///////////////////////////////////////////////////////////////////////////
 // Construction phase -- if we are constructing a tokenizer, this
-// phase builds up an internal token DFA.
+// phase builds up an internal token Dfa.
 
 fn construct(grammar: &mut Grammar, match_block: MatchBlock) -> NormResult<()> {
     let MatchBlock {
@@ -334,10 +328,10 @@ fn construct(grammar: &mut Grammar, match_block: MatchBlock) -> NormResult<()> {
         precedences.push(Precedence(match_entry.precedence));
         match match_entry.match_literal {
             TerminalLiteral::Quoted(ref s) => {
-                regexs.push(re::parse_literal(&s));
+                regexs.push(re::parse_literal(s));
             }
             TerminalLiteral::Regex(ref s) => {
-                match re::parse_regex(&s) {
+                match re::parse_regex(s) {
                     Ok(regex) => regexs.push(regex),
                     Err(error) => {
                         let literal_span = spans[&match_entry.match_literal];
@@ -353,13 +347,11 @@ fn construct(grammar: &mut Grammar, match_block: MatchBlock) -> NormResult<()> {
 
     let dfa = match dfa::build_dfa(&regexs, &precedences) {
         Ok(dfa) => dfa,
-        Err(DFAConstructionError::NFAConstructionError { index, error }) => {
+        Err(DfaConstructionError::NfaConstructionError { index, error }) => {
             let feature = match error {
                 NamedCaptures => r#"named captures (`(?P<foo>...)`)"#,
                 NonGreedy => r#""non-greedy" repetitions (`*?` or `+?`)"#,
-                WordBoundary => r#"word boundaries (`\b` or `\B`)"#,
-                LineBoundary => r#"line boundaries (`^` or `$`)"#,
-                TextBoundary => r#"text boundaries (`^` or `$`)"#,
+                LookAround => r#"all boundaries like `\b` or `\B` or `^` or `$`"#,
                 ByteRegex => r#"byte-based matches"#,
             };
             let literal = &match_entries[index.index()].match_literal;
@@ -369,7 +361,7 @@ fn construct(grammar: &mut Grammar, match_block: MatchBlock) -> NormResult<()> {
                 feature
             )
         }
-        Err(DFAConstructionError::Ambiguity { match0, match1 }) => {
+        Err(DfaConstructionError::Ambiguity { match0, match1 }) => {
             let literal0 = &match_entries[match0.index()].match_literal;
             let literal1 = &match_entries[match1.index()].match_literal;
             // FIXME(#88) -- it'd be nice to give an example here
