@@ -436,7 +436,7 @@ impl<'input> Tokenizer<'input> {
         // This is the interesting case. To find the end of the code,
         // we have to scan ahead, matching (), [], and {}, and looking
         // for a suitable terminator: `,`, `;`, `]`, `}`, or `)`.
-        // Additionaly we had to take into account that we can encounter an character literal
+        // Additionally we had to take into account that we can encounter an character literal
         // equal to one of delimiters.
         let mut balance = 0; // number of unclosed `(` etc
         loop {
@@ -600,6 +600,11 @@ impl<'input> Tokenizer<'input> {
                     None => error(UnterminatedStringLiteral, idx0),
                 }
             }
+            Some(idx1) if matches!(self.lookahead, Some((idx2, c)) if idx1 == idx2 && is_identifier_start(c)) =>
+            {
+                self.bump();
+                self.identifierish(idx0)
+            }
             Some(idx1) => error(ExpectedStringLiteral, idx1),
             None => error(UnterminatedStringLiteral, idx0),
         }
@@ -638,6 +643,10 @@ impl<'input> Tokenizer<'input> {
             return Ok((idx0, Underscore, idx0 + 1));
         }
 
+        if word == "r#_" {
+            return Ok((idx0, Underscore, idx0 + 3));
+        }
+
         if word == "use" {
             let code_end = self.code(idx0, "([{", "}])")?;
             let code = &self.text[end..code_end];
@@ -650,9 +659,9 @@ impl<'input> Tokenizer<'input> {
             // is a `<` immediately afterwards
             KEYWORDS.iter()
                     .filter(|&&(w, _)| w == word)
-                    .map(|&(_, ref t)| t.clone())
+                    .map(|(_, t)| t.clone())
                     .next()
-                    .unwrap_or_else(|| {
+                    .unwrap_or({
                         match self.lookahead {
                             Some((_, '<')) => MacroId(word),
                             _ => Id(word),
