@@ -11,13 +11,13 @@ use self::Expectation::*;
 fn gen_test(input: &str, expected: Vec<(&str, Expectation)>) {
     // use $ to signal EOL because it can be replaced with a single space
     // for spans, and because it applies also to r#XXX# style strings:
-    let input = input.replace("$", "\n");
+    let input = input.replace('$', "\n");
 
     let tokenizer = Tokenizer::new(&input, 0);
     let len = expected.len();
     for (token, (expected_span, expectation)) in tokenizer.zip(expected.into_iter()) {
-        let expected_start = expected_span.find("~").unwrap();
-        let expected_end = expected_span.rfind("~").unwrap() + 1;
+        let expected_start = expected_span.find('~').unwrap();
+        let expected_end = expected_span.rfind('~').unwrap() + 1;
         println!("token: {:?}", token);
         match expectation {
             ExpectTok(expected_tok) => {
@@ -33,8 +33,8 @@ fn gen_test(input: &str, expected: Vec<(&str, Expectation)>) {
         }
     }
 
-    let tokenizer = Tokenizer::new(&input, 0);
-    assert_eq!(None, tokenizer.skip(len).next());
+    let mut tokenizer = Tokenizer::new(&input, 0);
+    assert_eq!(None, tokenizer.nth(len));
 }
 
 fn test(input: &str, expected: Vec<(&str, Tok)>) {
@@ -521,19 +521,23 @@ fn code_forgot_comma() {
 #[test]
 fn various_kinds_of_ids() {
     test(
-        "foo<T<'a,U,`Z*{}`>>",
+        "foo<T<'a,U,`Z*{}`,r#type,r#use>>",
         vec![
-            ("~~~                ", MacroId("foo")),
-            ("   ~               ", LessThan),
-            ("    ~              ", MacroId("T")),
-            ("     ~             ", LessThan),
-            ("      ~~           ", Lifetime("'a")),
-            ("        ~          ", Comma),
-            ("         ~         ", Id("U")),
-            ("          ~        ", Comma),
-            ("           ~~~~~~  ", Escape("Z*{}")),
-            ("                 ~ ", GreaterThan),
-            ("                  ~", GreaterThan),
+            ("~~~                             ", MacroId("foo")),
+            ("   ~                            ", LessThan),
+            ("    ~                           ", MacroId("T")),
+            ("     ~                          ", LessThan),
+            ("      ~~                        ", Lifetime("'a")),
+            ("        ~                       ", Comma),
+            ("         ~                      ", Id("U")),
+            ("          ~                     ", Comma),
+            ("           ~~~~~~               ", Escape("Z*{}")),
+            ("                 ~              ", Comma),
+            ("                  ~~~~~~        ", Id("r#type")),
+            ("                        ~       ", Comma),
+            ("                         ~~~~~  ", Id("r#use")),
+            ("                              ~ ", GreaterThan),
+            ("                               ~", GreaterThan),
         ],
     );
 }
